#
# wakaba/controller.rb
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: controller.rb,v 1.13 2002/09/10 11:07:27 katsu Exp $
#

require 'wakaba/error'


module Wakaba

  class Controller

    def initialize
      @identifier = nil
      @store = nil
      @interface = nil
      @session_map = nil
      @serialize = false
      @record = nil
    end

    attr_reader :identifier, :store, :interface, :serialize
    alias serialize? serialize


    def identifier=(identifier)
      raise TypeError, "already started" if @record
      @identifier = identifier
    end

    def store=(store)
      raise TypeError, "already started" if @record
      @store = store
    end

    def interface=(interface)
      raise TypeError, "already started" if @record
      @interface = interface
    end

    def serialize=(flag)
      raise TypeError, "already started" if @record
      @serialize = !!flag
      flag
    end


    # interface for Context

    def save_session
      raise NotImplementedError, "yet not implemented"
    end

    def finale
      raise TypeError, "is not started" unless @record
      @record.alive = false
    end



    # main flow.

    def start

      # start interface.
      @interface.start
      @store.now = @interface.starttime

      # receive state infomation.
      index = @interface.index
      client_key = @interface.client_key
      session_name = @interface.session_name
      action = @interface.action
      context = nil

      index = nil unless index and client_key

      begin

        # TODO: if @store == nil.

        # open a record.
        @store.open(index, @serialize) { |@record|

          # prepare.
          if index then
            @identifier.authorize(@record.identity,
                                  client_key, @record.server_key)
            session_map = SessionMap.new(@record.object)
          else
            session_map = SessionMap.new
          end

          context = Context.new(self, @interface, session_map,
                                session_name, action)
          context.header_out['Content-Type'] = 'text/html'
          context.header_out['Expires'] = @interface.starttime

          unless index then
            # instanciate a new session.
            root = yield(context)
            session_map.hash[''] = root
          end

          # start!
          session = session_map.lookup(session_name)
          @record.alive = true
          session.dispatch context

          # finish.
          @record.object = session_map.hash
          client_key, server_key = @identifier.make_key_pair(@record.identity)
          @record.server_key = server_key
        }

        # send result.
        if @record.alive? then
          index = @record.index
        else
          index, client_key = nil
        end
        @interface.out context, index, client_key

      rescue AccessError => e

        @interface.access_error e

      end

    end

  end

end
