#
# erb.rb - ERb with Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: erb.rb,v 1.11 2002/07/29 13:02:55 katsu Exp $
#

require 'wakaba/pbinding'
require 'erb/compile'


module Wakaba

  class ERbViewBinding < ProtectedBinding

    private

    def import(filename)
      src = File.open(filename) { |f| f.read }
      src.untaint
      script = ERbCompiler.new.compile(src)
      b = class << self
            TOPLEVEL_BINDING
          end
      eval script, b, filename
    end

  end


  class ERbViewEval < ProtectedEval

    def context=(c)
      @parameters['Context'] = c
    end

    private

    def new_binding
      ERbViewBinding.new
    end

  end


  class ERbView

    def initialize(filename)
      @filename = filename
      @eval = ERbViewEval.new
    end

    attr_reader :filename

    def []=(key, value)
      @eval[key] = value
    end

    private

    def pre_script
      nil
    end

    def post_script
      nil
    end

    public

    def run(context)
      src = File.open(filename) { |f| f.read }
      src.untaint
      c = ERbCompiler.new
      pre_cmd = [
        "def print(*args); Context.print(*args); end",
        "def printf(*args); Context.printf(*args); end",
        "def puts(*args); Context.puts(*args); end",
        "def putc(*args); Context.putc(*args); end",
        "def href(*args); Context.href(*args); end",
      ]
      s = pre_script
      pre_cmd.concat s if s
      c.pre_cmd = pre_cmd
      s = post_script
      c.post_cmd = s if s
      script = c.compile(src)
      @eval.context = context
      @eval.eval script, @filename
    end

  end

end
