#
# wakaba/store.rb
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: identifier.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'digest/md5'
require 'wakaba/error'


module Wakaba

  class Identifier

    def make_key_pair(record_identity)
      raise AbstractMethod
      # [ client_key, server_key ]
    end

    def deny
      raise AccessDenied, 'session authorization failed'
    end
    private :deny

    def authorize(record_identity, client_key, server_key)
      # authorization always fails by default.
      deny
    end

  end




  class SimpleIdentifier < Identifier

    def initialize(secret_key = nil)
      @secret_key = secret_key || ''
    end

    def md5_digest(str)
      # MD5 digest is 128 bits, so base64 encoded digest always consists
      # of 25 characters (including linebreak) and ends with "==\n".
      [ Digest::MD5.digest(str) ].pack('m')[0, 22].tr('+/', '_-')
    end
    private :md5_digest

    Salt = "B6f90YyFlS7DkK4gwhXOpi-1PZnArbJd2EIQMzNUm8Wj_CausctTG5voqxVeLH3R"

    def make_key_pair(identity)
      client = ''
      16.times { client << Salt[rand(Salt.size)] }
      server = md5_digest("#{identity}:#{@secret_key}:#{client}")
      [ client, server ]
    end

    def authorize(identity, client_key, server_key)
      key = md5_digest("#{identity}:#{@secret_key}:#{client_key}")
      deny unless key == server_key
    end

  end

end
