#
# cgi.rb - wakaba with cgi.rb
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: cgi.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'cgi'
require 'wakaba/interface'


module Wakaba

  class CGIInterface < Interface

    class ExtendedCGI < CGI

      def initialize(*args)
        super
        def self.path_info
          path = super
          if iis? then
            name = script_name
            path = path[name.size..-1] if path[0,name.size] == name
          end
          path || ''
        end
      end

      def iis?
        /IIS/n =~ server_software
      end

      def script_absolute_uri
        host = server_name
        port = server_port
        host = host + ":#{port}" unless port == 80
        protocol = server_protocol.split('/', 2)[0]
        protocol.downcase!
        protocol + '://' + host + script_name
      end

      def error(status, message, extra_options)
        options = {}
        options['type'] = 'text/html'
        options['status'] = HTTP_STATUS[status] || status
        options.update extra_options
        title = HTTP_STATUS[status] || status
        name = title.sub(/\A\d+\s*/, '')
        out(options) {
          "<!DOCTYPE html PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n" \
          "<html><head><title>#{CGI.escapeHTML(title)}</title></head>\n" \
          "<body><h1>#{CGI.escapeHTML(name)}</h1>\n" \
          "<p>#{CGI.escapeHTML(message)}</p>\n" \
          "</body></html>\n"
        }
      end

    end



    private

    def moved_permanently(uri)
      uri = @cgi.script_absolute_uri + uri
      @cgi.error('MOVED', "The requested resource has moved to #{uri}.",
                 { 'Location' => uri })
    end


    def forbidden
      uri = @cgi.script_name + @cgi.path_info
      @cgi.error('FORBIDDEN',
                 "You don't have permission to access #{uri} on this server.",
                 {})
    end


    def relative_uri(to)
      from = @cgi.path_info.split('/', -1)
      from.pop
      to = to.split('/', -1)
      from.each_with_index { |i,n|
        unless i == to[n] then
          return '../' * (from.size - n) + to[n..-1].join('/')
        end
      }
      s = to[from.size..-1].join('/')
      s = '.' if s.empty?
      s
    end


    public

    def start
      @cgi = ExtendedCGI.new
      path = @cgi.path_info
      if path.empty? then
        moved_permanently '/'
        exit
      end
      @session_name, @action = path.split(/\/(?=[^\/]*\z)/, 2)
      @index, @client_key = @cgi.cookies['session']
    end

    attr_reader :index, :client_key, :session_name, :action


    def out(context, index, client_key)
      if index and client_key then
        cookie = CGI::Cookie.new('name' => 'session',
                                 'value' => [ index, client_key ],
                                 'path' => @cgi.script_name)
      else
        cookie = CGI::Cookie.new('name' => 'session',
                                 'path' => @cgi.script_name,
                                 'expires' => Time.now)
      end
      header = {}
      context.header_out.each_out { |k,v| header[k] = v }
      header['cookie'] = cookie
      @cgi.out(header) { context.body_out }
    end


    def access_error(error)
      forbidden
    end


    def cgienv(key)
      ENV[key]
    end


    def header_in(key)
      key = key.upcase
      key.tr! '-', '_'
      ENV[key]
    end


    def param
      @cgi.params
    end


    def href(session_name, action)
      if session_name.empty? then
        relative_uri "/#{action}"
      else
        relative_uri "/#{session_name}/#{action}"
      end
    end


    def base_uri
      n = @cgi.path_info.count('/')
      if n == 0 then
        '.'
      else
        '../' * (n - 1) + '..'
      end
    end

  end

end
