#
# pstore.rb - Wakaba with PStore
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: pstore.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'pstore'
require 'wakaba/store'



module Wakaba

  class PStore < Store

    def initialize(filename, *args)
      super(*args)
      @pstore = ::PStore.new(filename)
    end

    private

    def do_open(index)
      record = nil
      @pstore.transaction {
        record = @pstore[index]
        @pstore.abort
      }
      object, server_key, version, ctime = record
      unless object and server_key then
        raise StoreError, ""
      end
      record = Record.new(index, object, server_key, version, ctime)
      yield record
      @pstore.transaction {
        if record.alive? then
          @pstore[index] = [
            record.object, record.server_key, record.version, record.ctime
          ]
        else
          @pstore.delete index
        end
      }
    end

    def do_open_ex(index)
      @pstore.transaction {
        record = @pstore[index]
        object, server_key, version, ctime = record
        unless object and server_key then
          raise StoreError, ""
        end
        record = Record.new(index, object, server_key, version, ctime)
        yield record
        if record.alive? then
          @pstore[index] = [
            record.object, record.server_key, record.version, record.ctime
          ]
        else
          @pstore.delete index
        end
      }
    end


    Source = "fYIS0ycHxh-3OLTnreU_5lktMRjAXE89QwP4G2ZJd7VpqDz1gW6KNbsmuoaFvCBi"

    def do_create
      index = nil
      @pstore.transaction {
        begin
          index = ''
          8.times { index << Source[rand(Source.size)] }
        end while @pstore.root? index
        @pstore[index] = [ nil, nil, version, now ]
      }
      record = Record.new(index, nil, nil, version, now)
      err = nil
      begin
        yield record
      rescue => err
        record.alive = false
      end
      @pstore.transaction {
        if record.alive? then
          @pstore[index] = [
            record.object, record.server_key, record.version, record.ctime
          ]
        else
          @pstore.delete index
        end
      }
      raise err, err.message, err.backtrace if err
    end

  end


end
