#
# strio.rb - tiny string output for environments missing stringio.so
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: strio.rb,v 1.1 2002/07/15 01:59:47 katsu Exp $
#

module Wakaba

  class StringIO

    def initialize(str = '')
      self.string = str.to_str
    end

    attr_reader :string

    def string=(str)
      @string = str.to_str
    end

    def write(str)
      @string.concat str
      str.size
    end

    def <<(str)
      @string.concat str
      self
    end

    def switch_defout
      oldstate = Thread.critical
      begin
        Thread.critical = true
        olddefout = $defout
        $defout = self
        yield
      ensure
        $defout = olddefout
        Thread.critical = oldstate
      end
    end
    private :switch_defout

    def print(*args)
      switch_defout { Kernel.print(*args) }
    end

    def printf(*args)
      switch_defout { Kernel.printf(*args) }
    end

    def puts(*args)
      switch_defout { Kernel.puts(*args) }
    end

    def putc(c)
      if c.is_a? String and c.size >= 1 then
        @string << c[0]
      else
        @string << (Integer(c) & 0xff)
      end
      c
    end

  end

end
