#!/usr/bin/ruby -Ke -T1
#
# quiz.rb - a simple example for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: quiz.rb,v 1.2 2002/09/10 11:07:27 katsu Exp $
#

require 'wakaba/wakaba'
require 'wakaba/interface/cgi'
require 'wakaba/erb'
require 'erb/erbu'

AppVersion = "1"
ViewDir = '.'     # you should modify here.



class QuizView < Wakaba::ERbView

  def initialize(filename)
    super ViewDir + '/' + filename + '.rhtml'
  end

  def pre_script
    [ 'extend ERbUtil' ]
  end
  private :pre_script

  def show(c)
    c.header_out['Content-Type'] = 'text/html; charset=euc-jp'
    run c
  end

end



class Quiz < Wakaba::Session

  Question = Struct.new(:question, :choice, :answer)

  QuestionList = [
    Question.new( '', [ '', '', '', 'ȡ' ], 1 ),
    Question.new( '', [ '', '', '', 'ȡ' ], 2 ),
    Question.new( '', [ '', '', '', 'ȡ' ], 3 ),
    Question.new( 'Ĥơ', [ '', '', '', 'ȡ' ], 4 ),
  ]


  def initialize
    @answer = []
  end


  private

  def next_question(c)
    no = @answer.size + 1
    q = QuestionList[@answer.size]
    if no == QuestionList.size then
      method = :on_result
    else
      method = :on_answer
    end
    view = QuizView.new('question')
    view['next_uri'] = c.href(self, method)
    view['no'] = no
    view['question'] = q.question
    view['choice'] = q.choice
    view.show c
  end


  def save_answer(c)
    answer = c.param['answer'][0].to_i
    no = c.param['no'][0].to_i
    deny unless no == @answer.size + 1
    deny unless answer >= 1 and answer <= QuestionList[no - 1].choice.size
    @answer.push answer
  end


  public

  def welcome(c)
    @answer = []
    next_question c
  end


  def on_answer(c)
    save_answer c
    next_question c
  end


  def on_result(c)
    save_answer c
    dany unless @answer.size == QuestionList.size
    result = []
    @answer.each_with_index { |i,n|
      result.push [ i == QuestionList[n].answer, i ]
    }

    view = QuizView.new('result')
    view['next_uri'] = c.href(self, :welcome)
    view['result'] = result
    view.show c

    c.finale
  end


end




c = Wakaba::Controller.new
c.interface = Wakaba::CGIInterface.new
c.store = Wakaba::EchoStore.new(AppVersion, 900)        # 15 min.
c.identifier = Wakaba::SimpleIdentifier.new('hoge')
c.start { Quiz.new }
