#
# asserttracecall.rb - assert for call trace
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: asserttracecall.rb,v 1.1 2002/07/12 11:46:08 katsu Exp $
#

require 'runit/testcase'


module RUNIT

  module Assert

    def assert_trace_call(*args)
      check_failed = false
      set_trace_func proc { |event,file,line,id,binding,klass|
        unless args.empty? then
          if event == 'call' or event == 'c-call' then
            obj, method, checks = args.first
            method = method.to_s.intern unless method.is_a? Symbol
            if obj.equal? eval('self', binding) and method == id then
              args.shift
              if checks then
                checks.each { |var,expected|
                  actual = eval(var, binding)
                  unless expected == actual then
                    check_failed = "method:<#{method}> cond:<#{var}> expected:<#{expected}> but was:<#{actual}>"
                    throw :__assert_trace_call_tag__
                  end
                }
              end
            end
          end
        end
      }
      exception_raised = false
      begin
        ret = catch(:__assert_trace_call_tag__) { yield }
      rescue Exception
        exception_raised = true
        raise
      ensure
        set_trace_func nil
        unless exception_raised then
          begin
            if check_failed then
              raise_assertion_error check_failed
            elsif not args.empty? then
              raise_assertion_error "method <#{args[0][1]}> is not called"
            end
          rescue AssertionFailedError => e
            raise e, e.message, e.backtrace[2..-1]
          end
        end
      end
      ret
    end

  end

end




class TestAssertTraceCall < RUNIT::TestCase

  class Foo
    def foo(a) ; end
    def bar ; end
    def hoge ; bar ; end
    def err ; raise ; end
  end

  def setup
    @f = Foo.new
  end

  def test_no_vars
    assert_no_exception {
      assert_trace_call([@f, :bar]) { @f.bar }
    }
  end

  def test_no_vars_fail
    e = assert_exception(RUNIT::AssertionFailedError) {
      assert_trace_call([@f, :bar]) { @f.foo 1 }
    }
    assert_equal 'method <bar> is not called', e.message
    assert_equal("#{__FILE__}:#{__LINE__-3}:in `test_no_vars_fail'",
                 e.backtrace[0])
  end

  def test_vars
    assert_no_exception {
      assert_trace_call([@f, :foo, {'a'=>1}]) { @f.foo 1 }
    }
  end

  def test_vars_fail
    e = assert_exception(RUNIT::AssertionFailedError) {
      assert_trace_call([@f, :foo, {'a'=>2}]) { @f.foo 1 }
    }
    assert_equal "method:<foo> cond:<a> expected:<2> but was:<1>", e.message
    assert_equal("#{__FILE__}:#{__LINE__-3}:in `test_vars_fail'",
                 e.backtrace[0])
  end

  def test_repeat
    assert_no_exception {
      assert_trace_call([@f, :bar]) { @f.bar ; @f.bar }
    }
  end

  def test_order
    assert_no_exception {
      assert_trace_call([@f, :bar], [@f, :foo]) { @f.bar ; @f.foo 1 }
    }
  end

  def test_order_fail
    e = assert_exception(RUNIT::AssertionFailedError) {
      assert_trace_call([@f, :bar], [@f, :foo]) { @f.foo 1 ; @f.bar }
    }
    assert_equal 'method <foo> is not called', e.message
    assert_equal("#{__FILE__}:#{__LINE__-3}:in `test_order_fail'",
                 e.backtrace[0])
  end

  def test_nest
    assert_no_exception {
      assert_trace_call([@f, :bar]) { @f.hoge }
    }
  end

  def test_nest_order
    assert_no_exception {
      assert_trace_call([@f, :hoge], [@f, :bar]) { @f.hoge }
    }
  end

  def test_raise
    assert_exception(RuntimeError) {
      assert_trace_call([@f, :err]) { @f.err }
    }
  end

  def test_argerror
    assert_exception(ArgumentError) {
      assert_trace_call([@f, :foo]) { @f.foo }
    }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
