#
# testidentifier.rb - test cases for Wakaba
#
#   Copyright (C) UENO Katsuhiro 2002
#
# $Id: testidentifier.rb,v 1.1 2002/09/10 11:07:27 katsu Exp $
#

require 'runit/testcase'
require 'asserttracecall'
require 'wakaba/identifier'


class TestIdentifier < RUNIT::TestCase

  def setup
    @i = Wakaba::Identifier.new
  end

  def test_make_key_pair
    assert_exception(Wakaba::AbstractMethod) { @i.make_key_pair 'hoge' }
  end

  def test_authorize
    assert_exception(Wakaba::AccessDenied) { @i.authorize 'foo','bar','baz' }
  end

  def test_deny
    e = assert_exception(Wakaba::AccessDenied) { @i.__send__ :deny }
    assert_equal 'session authorization failed', e.message
  end

end




class TestSimpleIdentifier < RUNIT::TestCase

  def setup
    @i = Wakaba::SimpleIdentifier.new('hoge')
  end

  def md5_digest(str)
    [ Digest::MD5.digest(str) ].pack('m')[0, 22].tr('+/', '_-')
  end

  def test_make_key_pair
    client, server = @i.make_key_pair('fuga')
    assert_kind_of String, client
    assert_kind_of String, server
    assert_match client, /\A[0-9a-zA-Z_-]{16}\z/n
    assert_match server, /\A[0-9a-zA-Z_-]{22}\z/n
    assert_equal md5_digest("fuga:hoge:#{client}"), server
  end

  def test_make_key_pair_unique
    client, server = @i.make_key_pair('fuga')
    client2, server2 = @i.make_key_pair('fuga')
    assert_exception(RUNIT::AssertionFailedError) {
      assert_equal client, client2
    }
    assert_exception(RUNIT::AssertionFailedError) {
      assert_equal server, server2
    }
  end

  def test_make_key_pair_no_secret
    @i = Wakaba::SimpleIdentifier.new
    client, server = @i.make_key_pair('fuga')
    assert_equal md5_digest("fuga::#{client}"), server
  end

  def test_authorize
    client_key, server_key = @i.make_key_pair('fuga')
    assert_no_exception { @i.authorize 'fuga', client_key, server_key }
  end

  def test_authorize_fail_1
    client_key, server_key = @i.make_key_pair('fuga')
    assert_exception(Wakaba::AccessDenied) {
      @i.authorize 'fuga', client_key + 'a', server_key
    }
  end

  def test_authorize_fail_2
    client_key, server_key = @i.make_key_pair('fuga')
    assert_exception(Wakaba::AccessDenied) {
      @i.authorize 'fuga', client_key, server_key + 'a'
    }
  end

  def test_authorize_fail_3
    client_key, server_key = @i.make_key_pair('fuga')
    assert_exception(Wakaba::AccessDenied) {
      @i.authorize 'fugaa', client_key, server_key
    }
  end

  def test_authorize_fail_4
    assert_exception(Wakaba::AccessDenied) {
      @i.authorize 'fuga', 'foo', 'bar'
    }
  end

  def test_authorize_no_secret
    @i = Wakaba::SimpleIdentifier.new
    client, server = @i.make_key_pair('fuga')
    assert_no_exception { @i.authorize 'fuga', client, server }
  end

end




load "#{File.dirname($0)}/runtest.rb" if __FILE__ == $0
